@@ -18,7 +18,7 @@ sub init
 	my $self = shift;
 	$self->SUPER::init;
 
-	$self->{canonicalize_body_empty_lines} = 0;
+	$self->{canonicalize_body_buf} = "";
 }
 
 sub canonicalize_header
@@ -39,7 +39,7 @@ sub canonicalize_header
 sub canonicalize_body
 {
 	my $self = shift;
-	my ($multiline) = @_;
+	my ($line) = @_;
 
 	#
 	# draft-allman-dkim-base-01.txt, section 3.4.3:
@@ -51,30 +51,19 @@ sub canonicalize_body
 	# (i.e. do not emit empty lines until a following nonempty line
 	# is found)
 	#
-	my $empty_lines = $self->{canonicalize_body_empty_lines};
-
-	if ( $multiline =~ s/^((?:\015\012)+)// )
-	{	# count & strip leading empty lines
-		$empty_lines += length($1)/2;
-	}
-
-	if (length($multiline) > 0)
+	if ($line eq "\015\012")
 	{
+		$self->{canonicalize_body_buf} .= $line;
+		$line = "";
+	}
+	else
+	{
+		$line = $self->{canonicalize_body_buf} . $line;
+		$self->{canonicalize_body_buf} = "";
 		$self->{canonicalize_body_started} = 1;
-		if ($empty_lines > 0)
-		{	# re-insert leading white if any nonempty lines exist
-			$multiline = ("\015\012" x $empty_lines) . $multiline;
-			$empty_lines = 0;
-		}
 	}
 
-	if ($multiline =~ s/((?:\015\012){2,})\z/\015\012/)
-	{	# count & strip trailing empty lines
-		$empty_lines += length($1)/2 - 1;
-	}
-
-	$self->{canonicalize_body_empty_lines} = $empty_lines;
-	return $multiline;
+	return $line;
 }
 
 sub finish_body
